    %% Example: Bus aggregation methodology
% Based on the paper: An improved Bus Aggregation Technique for Generating Network Equivalents, Di Shi et.al., 2012.
% Author: Andr Ortner
% Date: 22.10.2013

% Definitions
%{
bus = node
branch = line
cluster = group of nodes
zone = assigned group of countries
%}

clear all
clc

%% load input data

% load country list of numbers, abbreviation and names

run('load_input_data.m'); % load node and line list, and node_injections

load 'countrylist.mat'; % load countrylist with numbers, abbreviations and names

run('zone_numbering.m'); % insert zone numbers for line and nodelist

%%   PTDF calculation  

run('PTDF_calculation.m'); % calculate PTDF and line flows in the full grid


%% clustering process

clusterintensity = 0.5; % what grid size will be aimed for. As a share of original grid size, ranging from 0 to 1.

run('clustering_process.m'); 

%% ----- Build reduced grid equivalent ----- %%

run('build_reduced_node_and_line_list.m'); % creat node and line list of the reduced grid

%% ----- Calculate reduced PTDF ----- %%

run('reduced_PTDF_calculation.m'); % derive reduced PTDF matrix based on reference set of injections

%% ----- Calculate the properties of the reduced grid ----- %%
%{
run('calculate_properties_reduced_grid.m'); % calculate properties reduced grid
% input grid size limited by available memory (Lambda)
%}
%% ----- Calculate the errors in inter-zonal caused by reduction

run('calculate_error_metric.m'); % calculate the average error of inter-zonal lines per zone

%% write excel file with the results

run('write_excel.m'); % write excel file with error and other properties of simulation
