disp('cluster process');
 c = clock;
 disp(datestr(datenum(c(1),c(2),c(3),c(4),c(5),c(6))));

 % clustering process, lines are selected based on reactance
 % only lines that do not exceed their capacity are allowed to be clustered
exceed_counter = zeros(num_lines,1);
max_exceed = zeros(num_lines,1);

for i = 1:num_lines
   exceed_counter(i) = length(P_flow(i,abs(P_flow(i,:))>mpc.branch(i,13)));  % count number of times exceed line capacity
   max_exceed(i) = max(P_flow(i,:))-mpc.branch(i,13); % count maximum excession
end

mpc.branch(:,20) = mpc.branch(:,12); %condition value based on reactance
max_treshold = max(mpc.branch(:,20)); %normalize condition index from 0 to 1
mpc.branch(:,20) = mpc.branch(:,20)/max_treshold; %normalize condition index from 0 to 1

number_zones = max(mpc.bus(:,6));
num_nodes_in_zone = zeros(number_zones,1);
num_clusters_in_zone = zeros(number_zones,1);
target_num_clusters_in_zone = zeros(number_zones,1);
num_lines_aggregate_zone = zeros(number_zones,1);

number_nodes = length(mpc.bus(:,1));
num_lines = length(mpc.branch(:,1));
mpc.bus(:,7) = 0;
g=0;

% assign cluster ID's based on clustering condition, clustering process gets repeated while enheightening the condition treshold
% until desired zone size is reached

for j=1:number_zones
    
    zone_nodes_IND = find(mpc.bus(:,6) ==j); %
    zone_nodes = mpc.bus(zone_nodes_IND,:); %selection of only nodes of the current zone
    num_nodes_in_zone(j,1) = length(mpc.bus(mpc.bus(:,6) ==j));  % count number of nodes in zone
    
    zone_branches_IND = find(mpc.branch(:,21)==j & mpc.branch(:,22)==j); % selection of only branches IN the current zone
    zone_branches = mpc.branch(zone_branches_IND,:);  % select number of lines in zone 
    num_branches_zone = length(zone_branches(:,1)); % count number of lines in zone
    zone_exceed_counter = exceed_counter(zone_branches_IND,:); % select the exceed_counter for lines in the zone
    line_IND_not_remove = find(zone_exceed_counter > 0); % if line flow exceeds capacity, do not cluster
    num_lines_not_remove = length(line_IND_not_remove); % count number of lines
    
    target_num_clusters_in_zone(j,1) = clusterintensity*num_nodes_in_zone(j,1); % number of clusters in zone is initally equal to the number of nodes 
    num_lines_aggregate_zone(j,1) = num_nodes_in_zone(j,1) - ceil(target_num_clusters_in_zone(j,1));
    if num_lines_aggregate_zone(j,1) > num_branches_zone - num_lines_not_remove  % if the targeted size is smaller than possible (because some lines cannot be clustered), see next line
        num_lines_aggregate_zone(j,1) = num_branches_zone - num_lines_not_remove;  % then the number of lines aggregated is set at the maximum possible value
    end
    
    if ~isempty(zone_nodes(:,1)) % if the zone contains nodes
    zone_branches(line_IND_not_remove,:) = []; % select lines eligible for clustering
    zone_branches = sortrows(zone_branches,20);  % sort based on condition value
          for i=num_lines_aggregate_zone(j,1):-1:1 % start aggregating on condition value from the bottom, highest condition value
            % select which lines to delete
                    %create clustersID's  
                    from_IND = find(mpc.bus(:,11) == zone_branches(i,18)); %find the from_node of a branch in the zone nodes
                    to_IND = find(mpc.bus(:,11) == zone_branches(i,19));  %find the to_node of a branch in the zone nodes
                    
                    if  mpc.bus(from_IND,7) ~= 0 && mpc.bus(to_IND,7) ~= 0 % if both nodes from a branch already have a cluster number, the to_node gets the number of the from_node
                    replace_cluster = mpc.bus(to_IND,7);
                    for l=1:number_nodes
                        if mpc.bus(l,7) == replace_cluster;  
                            mpc.bus(l,7) = mpc.bus(from_IND,7);
                        end
                    end
                    
                elseif mpc.bus(from_IND,7) ~= 0 % if only the from_node already has a cluster number, the to_node gets the number of the from_node
                    mpc.bus(to_IND,7)= mpc.bus(from_IND,7);
                elseif mpc.bus(to_IND,7) ~= 0 % if only the to_node already has a cluster number, the from_node gets the number of the to_node
                    mpc.bus(from_IND,7) = mpc.bus(to_IND,7);
                else % if none of the nodes of a branch has a cluster number, the cluster counter is going up 1, and both from_node and to_node get this cluster number assigned    
                    g=g+1;
                    mpc.bus(from_IND,7) =g;
                    mpc.bus(to_IND,7) =g;
                    end  
              end   
            end  
        
        %number all unnumbered nodes in the zone
        for b=1:number_nodes
            if mpc.bus(b,6) == j
                if mpc.bus(b,7) == 0
                  g = g+1;
                  mpc.bus(b,7) = g;         
                end         
            end 
        end
end

%renumber from 1 to num_clusters, as the previous numbering process may have skipped numbers in the process
replace_cluster = zeros(number_nodes,1);
num_clusters = length(unique(mpc.bus(:,7)));
for i=1:num_clusters 
rep_clus = find(mpc.bus(:,7) == max(mpc.bus(:,7)));
replace_cluster(rep_clus,1) = i;
mpc.bus(rep_clus,7)=0;
end
mpc.bus(:,7) = replace_cluster(:,1);

% create matrix describing which cluster belongs to which node
mpc.clusterNodeSumID = zeros(num_clusters,number_nodes); %matrix containing the indices of node injections in mpc.bus of zone i of the reduced grid
for i=1:num_clusters, 
    mpc.clusterNodeSumID(i,:) = ismember(mpc.bus(:,7),i)'; 
end;
